/**
 * JavaScript for Weather - includes current conditions and a 7-day forecast.
 *
 * The geo coordinates are passed from PHP via the localized variable "twp_geo".
 */

// Use coordinates passed from PHP (or default to 43.75, -87.71 if missing).
let lat = (typeof twp_geo !== 'undefined' && twp_geo.lat) ? parseFloat(twp_geo.lat) : 43.75;
let lon = (typeof twp_geo !== 'undefined' && twp_geo.lon) ? parseFloat(twp_geo.lon) : -87.71;

/**
 * Convert ISO date/time to a local Date object.
 */
function parseISOTime(isoTime) {
  return new Date(isoTime);
}

/**
 * Compute sunrise, sunset, and moon phase using SunCalc.
 */
function computeSolarData(dateObj, lat, lon) {
  const times = SunCalc.getTimes(dateObj, lat, lon);
  const sunrise = times.sunrise.toLocaleTimeString([], { hour: '2-digit', minute: '2-digit' });
  const sunset = times.sunset.toLocaleTimeString([], { hour: '2-digit', minute: '2-digit' });
  
  const moonIllum = SunCalc.getMoonIllumination(dateObj);
  let moonPhase = "";
  if (moonIllum.phase < 0.03 || moonIllum.phase > 0.97) {
    moonPhase = "New Moon";
  } else if (moonIllum.phase < 0.25) {
    moonPhase = "Waxing Crescent";
  } else if (moonIllum.phase < 0.27) {
    moonPhase = "First Quarter";
  } else if (moonIllum.phase < 0.50) {
    moonPhase = "Waxing Gibbous";
  } else if (moonIllum.phase < 0.53) {
    moonPhase = "Full Moon";
  } else if (moonIllum.phase < 0.75) {
    moonPhase = "Waning Gibbous";
  } else if (moonIllum.phase < 0.77) {
    moonPhase = "Last Quarter";
  } else {
    moonPhase = "Waning Crescent";
  }
  
  return {
    sunrise: sunrise,
    sunset: sunset,
    moonPhase: moonPhase
  };
}

/**
 * NOAA wind chill formula.
 */
function computeWindChill(tempF, windMph) {
  if (tempF >= 30 || windMph < 3) {
    return null;
  }
  const wc = 35.74 +
             0.6215 * tempF -
             35.75 * Math.pow(windMph, 0.16) +
             0.4275 * tempF * Math.pow(windMph, 0.16);
  return Math.round(wc);
}

/**
 * Extract numeric wind speed from a string.
 */
function parseWindSpeed(windStr) {
  const match = windStr.match(/\d+/);
  return match ? parseInt(match[0], 10) : 0;
}

/**
 * Toggle accordion open/closed.
 */
function toggleAccordion(header, content) {
  const isActive = header.classList.contains("active");
  document.querySelectorAll(".accordion-header").forEach(h => h.classList.remove("active"));
  document.querySelectorAll(".accordion-content").forEach(c => {
    c.classList.remove("expanded");
    c.style.maxHeight = null;
  });
  if (!isActive) {
    header.classList.add("active");
    content.classList.add("expanded");
    content.style.maxHeight = content.scrollHeight + "px";
  }
}

/**
 * Fetch JSON from a URL with custom headers.
 */
async function fetchNWS(url) {
  const response = await fetch(url, {
    headers: {
      "User-Agent": "(Your referrer info is set via plugin settings)",
      "Accept": "application/geo+json"
    },
    cache: "no-store"
  });
  if (!response.ok) {
    throw new Error("NWS fetch error: " + response.status);
  }
  return response.json();
}

/**
 * Aggregate grid data over a given time window.
 */
function aggregateGridData(gridValues, windowStart, windowEnd, type = "average") {
  const relevant = gridValues.filter(v => {
    const [startStr, durationStr] = v.validTime.split("/");
    const startTime = new Date(startStr);
    const match = durationStr.match(/PT(\d+)H/);
    const hours = match ? parseInt(match[1], 10) : 1;
    const endTime = new Date(startTime.getTime() + hours * 3600000);
    return endTime > windowStart && startTime < windowEnd;
  });
  if (!relevant.length) return null;
  const values = relevant.map(r => r.value).filter(val => val !== null);
  if (!values.length) return null;
  if (type === "average") {
    const sum = values.reduce((a, b) => a + b, 0);
    return Math.round(sum / values.length);
  } else if (type === "max") {
    return Math.max(...values);
  }
  return null;
}

/**
 * Load and display the current temperature.
 */
async function loadCurrentTemperature() {
  try {
    const pointData = await fetchNWS(`https://api.weather.gov/points/${lat},${lon}`);
    const stationsUrl = pointData.properties.observationStations;
    const stationsData = await fetchNWS(stationsUrl);
    if (!stationsData.features || !stationsData.features.length) {
      document.getElementById("current-temp").textContent = "No nearby stations found.";
      return;
    }
    const stationId = stationsData.features[0].properties.stationIdentifier;
    const obsUrl = `https://api.weather.gov/stations/${stationId}/observations/latest`;
    const obsData = await fetchNWS(obsUrl);
    const celsius = obsData.properties?.temperature?.value;
    if (celsius === null || celsius === undefined) {
      document.getElementById("current-temp").textContent = "Temperature not available.";
      return;
    }
    const fahrenheit = (celsius * 9 / 5) + 32;
    document.getElementById("current-temp").textContent = "Temperature: " + Math.round(fahrenheit) + "\u00B0F";
  } catch (err) {
    document.getElementById("current-temp").textContent = "Error loading temperature.";
    console.error(err);
  }
}

/**
 * Load and display the 7-day forecast.
 */
async function loadDetailedForecast() {
  try {
    const pointData = await fetchNWS(`https://api.weather.gov/points/${lat},${lon}`);
    const forecastUrl = pointData.properties.forecast;
    const gridUrl = pointData.properties.forecastGridData;
    const [forecastData, gridData] = await Promise.all([
      fetchNWS(forecastUrl),
      fetchNWS(gridUrl)
    ]);
    const rhArray = gridData.properties.relativeHumidity ? gridData.properties.relativeHumidity.values : [];
    const popArray = gridData.properties.probabilityOfPrecipitation ? gridData.properties.probabilityOfPrecipitation.values : [];
    const gustArray = gridData.properties.windGust ? gridData.properties.windGust.values : [];
    const periods = forecastData.properties.periods;
    let html = "";
    periods.forEach((period, i) => {
      const headerId = "header-" + i;
      const contentId = "content-" + i;
      const startLocal = parseISOTime(period.startTime);
      const endLocal   = parseISOTime(period.endTime);
      const avgHumidity = aggregateGridData(rhArray, startLocal, endLocal, "average");
      const avgPop      = aggregateGridData(popArray, startLocal, endLocal, "average");
      const maxGust     = aggregateGridData(gustArray, startLocal, endLocal, "max");
      const tempF = period.temperature;
      const windVal = parseWindSpeed(period.windSpeed);
      // Use period.name (not the short forecast) to determine day vs. night.
      let extraHeaderClass = period.name.toLowerCase().includes("night") ? " night-label" : " day-label";
      let headingString = period.name + ": " + tempF + "\u00B0F " +
                          "<i class='" + ( 
                              period.shortForecast.toLowerCase().includes("rain") ? "wi wi-rain" :
                              period.shortForecast.toLowerCase().includes("cloudy") ? "wi wi-cloudy" :
                              period.shortForecast.toLowerCase().includes("snow") ? "wi wi-snow" :
                              period.shortForecast.toLowerCase().includes("thunder") ? "wi wi-thunderstorm" :
                              "wi wi-day-sunny" 
                          ) + "' style='margin-left:5px;'></i> " +
                          period.shortForecast + (avgPop && avgPop > 0 ? " (" + avgPop + "% chance)" : "");
      let detailedLine = "<div class='info-line'><i class='wi wi-cloudy-gusts'></i> " + period.detailedForecast + "</div>";
      const windDir = period.windDirection || "";
      const windSpeed = period.windSpeed || "";
      const windText = "Winds " + windDir + " " + windSpeed;
      let windLine = "<div class='info-line'><i class='wi wi-strong-wind'></i> <strong>Wind:</strong> " + windText.replace("Winds ", "") + "</div>";
      let feelsLike = computeWindChill(tempF, windVal);
      let feelsLikeLine = feelsLike !== null
        ? "<div class='info-line'><i class='wi wi-thermometer'></i> <strong>Feels Like:</strong> " + feelsLike + "\u00B0F</div>"
        : "";
      let humidityLine = avgHumidity !== null
        ? "<div class='info-line'><i class='wi wi-humidity'></i> <strong>Humidity:</strong> " + avgHumidity + "%</div>"
        : "";
      let popLine = "";
      if (avgPop !== null && avgPop > 0) {
        popLine = "<div class='info-line'><i class='wi wi-umbrella'></i> <strong>Chance:</strong> " + avgPop + "%</div>";
      }
      const dateOnly = new Date(startLocal.getFullYear(), startLocal.getMonth(), startLocal.getDate());
      const astro = computeSolarData(dateOnly, lat, lon);
      const headerClass = (i === 0) ? "accordion-header active" + extraHeaderClass : "accordion-header" + extraHeaderClass;
      const contentClass = (i === 0) ? "accordion-content expanded" : "accordion-content";
      html +=
        "<div class='accordion-item'>" +
          "<div class='" + headerClass + "' id='" + headerId + "' style='display:flex;justify-content:space-between;align-items:center;'>" +
            "<p style='margin:0;'>" + headingString + "</p>" +
            "<span class='accordion-toggle'>▶</span>" +
          "</div>" +
          "<div class='" + contentClass + "' id='" + contentId + "'>" +
            detailedLine +
            windLine +
            feelsLikeLine +
            humidityLine +
            (popLine ? "<hr/>" + popLine : "") +
            "<div class='info-line'><i class='wi wi-strong-wind'></i> <strong>Max Gust:</strong> " + (maxGust !== null ? maxGust + " mph" : "N/A") + "</div>" +
            "<hr/>" +
            "<div class='info-line'><i class='wi wi-sunrise'></i> <strong>Sunrise:</strong> " + astro.sunrise + "</div>" +
            "<div class='info-line'><i class='wi wi-sunset'></i> <strong>Sunset:</strong> " + astro.sunset + "</div>" +
            "<div class='info-line'><i class='wi wi-moon-alt-first-quarter'></i> <strong>Moon Phase:</strong> " + astro.moonPhase + "</div>" +
          "</div>" +
        "</div>";
    });
    document.getElementById("forecast-accordion").innerHTML = html;
    document.querySelectorAll(".accordion-header").forEach((header, idx) => {
      const content = document.getElementById("content-" + idx);
      header.addEventListener("click", function() {
        toggleAccordion(header, content);
      });
    });
    const first = document.getElementById("content-0");
    if (first) {
      first.style.maxHeight = first.scrollHeight + "px";
    }
  } catch (err) {
    console.error(err);
    document.getElementById("forecast-accordion").innerHTML = "<p>Error loading forecast data.</p>";
  }
}

/**
 * Initialize weather data loading after DOM is ready.
 */
document.addEventListener("DOMContentLoaded", async function() {
  await loadCurrentTemperature();
  await loadDetailedForecast();
});
