<?php
/*
Plugin Name: Time and Weather
Plugin URI: https://doyjo.com/time-and-weather
Description: Provides shortcodes to display the current time and weather (including a 7-day forecast) on your website, with customizable geo coordinates, time zone, and referrer info. A simple location lookup and an easy time zone selection make setup a breeze.
Version: 1.1
Author: Doyjo
Author URI: https://doyjo.com
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: time-and-weather
*/

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Enqueue Plugin Assets
 */
function taw_enqueue_assets() {
    // Enqueue local Weather Icons CSS.
    wp_enqueue_style(
        'weather-icons',
        plugin_dir_url( __FILE__ ) . 'css/weather-icons.min.css',
        array(),
        '2.0.12'
    );

    // Enqueue our custom CSS.
    wp_enqueue_style(
        'taw-custom-weather-css',
        plugin_dir_url( __FILE__ ) . 'css/custom-weather.css',
        array(),
        '1.0'
    );

    // Enqueue local SunCalc library.
    wp_enqueue_script(
        'suncalc',
        plugin_dir_url( __FILE__ ) . 'js/suncalc.min.js',
        array(),
        '1.8.0',
        true
    );

    // Enqueue our custom weather JavaScript.
    wp_enqueue_script(
        'taw-custom-weather-js',
        plugin_dir_url( __FILE__ ) . 'js/custom-weather.js',
        array('suncalc'),
        '1.0',
        true
    );

    // Pass geo coordinates to JavaScript.
    wp_localize_script( 'taw-custom-weather-js', 'taw_geo', array(
        'lat' => get_option( 'taw_lat', '43.75' ),
        'lon' => get_option( 'taw_lon', '-87.71' )
    ) );
}
add_action( 'wp_enqueue_scripts', 'taw_enqueue_assets' );

/**
 * [time] Shortcode - Displays the current time and date using the user-set time zone.
 */
function taw_time_shortcode() {
    $taw_timezone = get_option('taw_timezone');
    if ( ! $taw_timezone ) {
        $taw_timezone = 'America/Chicago'; // default if not set
    }
    try {
        $date = new DateTime("now", new DateTimeZone($taw_timezone));
    } catch(Exception $e) {
        $date = new DateTime("now", new DateTimeZone('UTC'));
    }
    $current_time = esc_html( $date->format('g:i:s A, l, F j, Y') );
    return "It is <strong>{$current_time}</strong>.";
}
add_shortcode( 'time', 'taw_time_shortcode' );

/**
 * [weather] Shortcode - Displays the weather container along with a link back to Doyjo.com.
 */
function taw_weather_shortcode() {
    ob_start(); ?>
    <div class="forecast-container">
        <!-- Current Conditions Section -->
        <div class="current-conditions" id="current-conditions">
            <h2><?php echo esc_html__( 'Current Conditions', 'time-and-weather' ); ?></h2>
            <p id="current-temp"><?php echo esc_html__( 'Loading current temperature...', 'time-and-weather' ); ?></p>
        </div>
        <!-- 7-Day Forecast Title -->
        <h2 class="forecast-title"><?php echo esc_html__( '7-Day Weather Forecast', 'time-and-weather' ); ?></h2>
        <!-- Placeholder for forecast details -->
        <div id="forecast-accordion"></div>
        <!-- Link back to Doyjo.com -->
        <p class="taw-powered-by" style="text-align:center; font-size:0.9em; margin-top:10px;">
            <a href="https://doyjo.com" target="_blank"><?php echo esc_html__( 'Powered by Doyjo', 'time-and-weather' ); ?></a>
        </p>
    </div>
    <?php
    return ob_get_clean();
}
add_shortcode( 'weather', 'taw_weather_shortcode' );

/**
 * Fetch Time, Date, and Weather Data from the Weather.gov API.
 * Uses custom coordinates and referrer info from the settings.
 */
function taw_fetch_time_date_weather() {
    $date = new DateTime("now", new DateTimeZone("UTC"));
    $formatted_date = $date->format('l, F jS, Y \a\t g:i:s A');

    // Retrieve user-defined coordinates.
    $lat = get_option( 'taw_lat', '43.75' );
    $lon = get_option( 'taw_lon', '-87.71' );
    $points_url = "https://api.weather.gov/points/{$lat},{$lon}";
    $weather = "Unable to retrieve detailed weather data.";

    // Retrieve referrer info.
    $ref_domain = get_option( 'taw_referrer_domain', 'YourDomain.com' );
    $ref_email  = get_option( 'taw_referrer_email', 'support@yourdomain.com' );
    $user_agent = sprintf('%s (%s)', $ref_domain, $ref_email);

    $resp = wp_remote_get( $points_url, array(
        'headers' => array('User-Agent' => $user_agent),
        'timeout' => 10
    ) );

    if ( ! is_wp_error( $resp ) ) {
        $data = json_decode( wp_remote_retrieve_body( $resp ), true );
        if ( ! empty( $data['properties']['forecast'] ) ) {
            $forecast_url = $data['properties']['forecast'];
            $forecast_resp = wp_remote_get($forecast_url, array(
                'headers' => array('User-Agent' => $user_agent),
                'timeout' => 10
            ) );
            if ( ! is_wp_error( $forecast_resp ) ) {
                $forecast_data = json_decode( wp_remote_retrieve_body( $forecast_resp ), true );
                if ( ! empty( $forecast_data['properties']['periods'] ) ) {
                    $periods = $forecast_data['properties']['periods'];
                    $detailed_forecast = array();
                    foreach ( $periods as $period ) {
                        $detailed_forecast[] = sprintf(
                            "%s: %s, %s\u00B0F. Humidity: %s%%. Winds: %s. %s",
                            $period['name'] ?? 'Unknown',
                            $period['shortForecast'] ?? 'No forecast',
                            $period['temperature'] ?? '?',
                            isset( $period['relativeHumidity']['value'] ) ? $period['relativeHumidity']['value'] : 'Unknown',
                            $period['windSpeed'] ?? 'No wind data',
                            $period['detailedForecast'] ?? 'No detailed forecast available'
                        );
                    }
                    $weather = implode( "\n\n", $detailed_forecast );
                }
            }
        }
    }

    return array(
        'date'    => $formatted_date,
        'weather' => $weather
    );
}

/**
 * ADMIN SETTINGS PAGE: Allow users to set custom geo coordinates, referrer info, time zone, and (optionally) a location.
 */

/**
 * Add a settings menu for the plugin.
 */
function taw_add_admin_menu() {
    add_options_page(
        'Time & Weather Settings',
        'Time & Weather',
        'manage_options',
        'taw_settings',
        'taw_settings_page'
    );
}
add_action( 'admin_menu', 'taw_add_admin_menu' );

/**
 * Render the settings page.
 */
function taw_settings_page() {
    // Prepare a list of allowed continents and time zones grouped by continent.
    $allowed = array("Africa", "America", "Antarctica", "Asia", "Atlantic", "Australia", "Europe", "Indian", "Pacific");
    $all_timezones = timezone_identifiers_list();
    $timezones_by_continent = array();
    foreach ($all_timezones as $tz) {
        if (strpos($tz, '/') !== false) {
            $parts = explode('/', $tz, 2);
            if (in_array($parts[0], $allowed)) {
                $timezones_by_continent[$parts[0]][] = $tz;
            }
        }
    }
    // Get the saved time zone and determine its continent.
    $tz_value = get_option('taw_timezone', '');
    $default_continent = 'America';
    if ($tz_value) {
        $parts = explode('/', $tz_value, 2);
        if (!empty($parts[0])) {
            $default_continent = $parts[0];
        }
    }
    ?>
    <div class="wrap">
      <h1><?php echo esc_html__( 'Time & Weather Settings', 'time-and-weather' ); ?></h1>
      <form method="post" action="options.php">
         <?php settings_fields( 'taw_settings_group' ); ?>
         <?php do_settings_sections( 'taw_settings' ); ?>
         <table class="form-table">
             <tr valign="top">
                 <th scope="row"><?php echo esc_html__( 'Location (City, Address, etc.)', 'time-and-weather' ); ?></th>
                 <td>
                     <input type="text" id="taw_location" name="taw_location" value="<?php echo esc_attr( get_option( 'taw_location', '' ) ); ?>" placeholder="<?php echo esc_attr__( 'e.g., Downtown Milwaukee', 'time-and-weather' ); ?>" style="width: 300px;" />
                     <button type="button" id="taw_lookup_button" class="button"><?php echo esc_html__( 'Lookup Coordinates', 'time-and-weather' ); ?></button>
                     <p class="description"><?php echo esc_html__( 'Enter a location and click "Lookup Coordinates" to automatically fill in latitude and longitude.', 'time-and-weather' ); ?></p>
                 </td>
             </tr>
             <tr valign="top">
                 <th scope="row"><?php echo esc_html__( 'Latitude', 'time-and-weather' ); ?></th>
                 <td><input type="text" id="taw_lat" name="taw_lat" value="<?php echo esc_attr( get_option( 'taw_lat', '43.75' ) ); ?>" /></td>
             </tr>
             <tr valign="top">
                 <th scope="row"><?php echo esc_html__( 'Longitude', 'time-and-weather' ); ?></th>
                 <td><input type="text" id="taw_lon" name="taw_lon" value="<?php echo esc_attr( get_option( 'taw_lon', '-87.71' ) ); ?>" /></td>
             </tr>
             <tr valign="top">
                 <th scope="row"><?php echo esc_html__( 'Time Zone', 'time-and-weather' ); ?></th>
                 <td>
                     <select id="taw_continent">
                         <?php 
                         foreach ($allowed as $continent) {
                             echo '<option value="' . esc_attr($continent) . '" ' . selected($continent, $default_continent, false) . '>' . esc_html($continent) . '</option>';
                         }
                         ?>
                     </select>
                     <input type="text" list="taw_timezone_datalist" id="taw_timezone" name="taw_timezone" value="<?php echo esc_attr($tz_value); ?>" placeholder="<?php echo esc_attr__( 'Select a time zone', 'time-and-weather' ); ?>" style="width: 300px;" />
                     <datalist id="taw_timezone_datalist"></datalist>
                     <p class="description"><?php echo esc_html__( 'Select your continent and then choose your time zone from the dropdown.', 'time-and-weather' ); ?></p>
                 </td>
             </tr>
             <tr valign="top">
                 <th scope="row"><?php echo esc_html__( 'Referrer Domain', 'time-and-weather' ); ?></th>
                 <td><input type="text" name="taw_referrer_domain" value="<?php echo esc_attr( get_option( 'taw_referrer_domain', 'YourDomain.com' ) ); ?>" /></td>
             </tr>
             <tr valign="top">
                 <th scope="row"><?php echo esc_html__( 'Referrer Email', 'time-and-weather' ); ?></th>
                 <td><input type="email" name="taw_referrer_email" value="<?php echo esc_attr( get_option( 'taw_referrer_email', 'support@yourdomain.com' ) ); ?>" /></td>
             </tr>
         </table>
         <?php submit_button(); ?>
      </form>
      <?php
      // Output the time zones by continent as a JSON object for JavaScript.
      ?>
      <script>
      var taw_timezones_by_continent = <?php echo json_encode($timezones_by_continent); ?>;
      jQuery(document).ready(function($){
         // Update the datalist for time zones based on the selected continent.
         function updateTimezoneDatalist(continent) {
             var optionsHtml = "";
             if (taw_timezones_by_continent[continent]) {
                 taw_timezones_by_continent[continent].forEach(function(tz) {
                     optionsHtml += "<option value='" + tz + "'>";
                 });
             }
             $("#taw_timezone_datalist").html(optionsHtml);
         }
         // On page load, update the datalist for the selected continent.
         var currentContinent = $("#taw_continent").val();
         updateTimezoneDatalist(currentContinent);
         
         // When the continent select changes, update the datalist.
         $("#taw_continent").on("change", function() {
             var selectedContinent = $(this).val();
             updateTimezoneDatalist(selectedContinent);
             $("#taw_timezone").val("");
         });
         
         // If a time zone is already selected, ensure the continent select matches.
         var tzValue = $("#taw_timezone").val();
         if(tzValue.indexOf("/") !== -1){
             var parts = tzValue.split("/");
             $("#taw_continent").val(parts[0]);
             updateTimezoneDatalist(parts[0]);
         }
         
         // Lookup Coordinates button functionality.
         $('#taw_lookup_button').on('click', function(){
            var location = $('#taw_location').val();
            if(!location){
               alert('Please enter a location.');
               return;
            }
            var url = 'https://nominatim.openstreetmap.org/search?format=json&q=' + encodeURIComponent(location);
            $.getJSON(url, function(data){
               if(data && data.length > 0){
                  var firstResult = data[0];
                  $('#taw_lat').val(firstResult.lat);
                  $('#taw_lon').val(firstResult.lon);
                  alert('Coordinates updated: Latitude ' + firstResult.lat + ', Longitude ' + firstResult.lon);
               } else {
                  alert('No results found for that location.');
               }
            }).fail(function(){
               alert('Error fetching geocode data.');
            });
         });
      });
      </script>
    </div>
    <?php
}

/**
 * Register the settings with sanitization callbacks.
 */
function taw_register_settings() {
    register_setting( 'taw_settings_group', 'taw_lat', 'sanitize_text_field' );
    register_setting( 'taw_settings_group', 'taw_lon', 'sanitize_text_field' );
    register_setting( 'taw_settings_group', 'taw_referrer_domain', 'sanitize_text_field' );
    register_setting( 'taw_settings_group', 'taw_referrer_email', 'sanitize_email' );
    register_setting( 'taw_settings_group', 'taw_location', 'sanitize_text_field' );
    register_setting( 'taw_settings_group', 'taw_timezone', 'sanitize_text_field' );
}
add_action( 'admin_init', 'taw_register_settings' );

/**
 * Add a custom "View details" link to the plugin row meta.
 */
function taw_plugin_row_meta( $links, $file ) {
    if ( strpos( $file, 'time-and-weather.php' ) !== false ) {
        $row_meta = array(
            'view_details' => '<a href="https://doyjo.com/time-and-weather" title="' . esc_attr__( 'View details on Doyjo.com', 'time-and-weather' ) . '" target="_blank">' . esc_html__( 'View details', 'time-and-weather' ) . '</a>',
        );
        $links = array_merge( $links, $row_meta );
    }
    return $links;
}
add_filter( 'plugin_row_meta', 'taw_plugin_row_meta', 10, 2 );
